@@ -261,6 +261,7 @@ func TestCSSInline(t *testing.T) {
 		{"flex:0 1 auto", "flex:initial"},
 		{"flex:1 1 auto", "flex:auto"},
 		{"flex:0 0 auto", "flex:none"},
+		{"flex:3 1 auto", "flex:3 1 auto"},
 		{"flex:0 0 5000%", "flex:0 0 5e3%"},
 		{"flex:initial", "flex:initial"},
 		{"flex:auto", "flex:auto"},
@@ -300,6 +301,7 @@ func TestCSSInline(t *testing.T) {
 		{"background:url('data:\\'\",text')", "background:url('data:\\'\",text')"},
 		{"margin:0 0 18px 0;", "margin:0 0 18px"},
 		{"z-index:1000", "z-index:1000"},
+		{"z-index:calc(1000)", "z-index:calc(1000)"},
 		//{"flex:0px", "flex:0q"}, // TODO
 		{"g:url('abc\\\ndef')", "g:url(abcdef)"},
 		{"url:local('abc\\\ndef')", "url:local(abcdef)"},
@@ -435,6 +437,7 @@ func TestCSSKeepCSS2(t *testing.T) {
 		{`margin:5000%`, `margin:5000%`},
 		{`margin:5000em`, `margin:5000em`},
 		{`color:transparent`, `color:transparent`},
+		{`background-color:transparent`, `background-color:transparent`},
 	}
 
 	m := minify.New()
